// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSC1M/16AI32SSC1M_Linux_1.x.x.x_DN/driver/device.c $
// $Rev: 53825 $
// $Date: 2023-10-04 16:30:11 -0500 (Wed, 04 Oct 2023) $

// 16AI32SSC1M: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model		Vendor	Device	SubVen	SubDev	type
	{ "16AI32SSC1M",	0x10B5, 0x9056, 0x10B5, 0x3592,	GSC_DEV_TYPE_16AI32SSC1M	},	// initial criteria
	{ NULL }
};



//*****************************************************************************
static int _channels_compute(dev_data_t* dev)
{
	int	errs	= 0;

	switch (dev->cache.gsc_bcfgr_32 & 0x30000)
	{
		case 0x20000:
		case 0x30000:
		default:
						errs	= 1;
						printf("Channel count is not known.\n");
						printf("Assuming a value of 32.\n");

						FALLTHROUGH;	/* fall through */

		case 0x00000:	dev->cache.channel_qty	= 32;	break;
		case 0x10000:	dev->cache.channel_qty	= 16;	break;
	}

	dev->cache.channels_max	= 32;
	return(errs);
}



//*****************************************************************************
static int _master_clock_compute(dev_data_t* dev)
{
	int	errs	= 0;

	switch (dev->cache.gsc_bcfgr_32 & 0xC0000)
	{
		case 0x40000:
		case 0x80000:
		case 0xC0000:
		default:
						errs	= 1;
						printf("Master clock frequency not known.\n");
						printf("Assuming a value of 64000000.\n");

						FALLTHROUGH;	/* fall through */

		case 0x00000:	dev->cache.master_clock	= 64000000L;	break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(AI32SSC1M_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_acar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ACAR));
		dev->vaddr.gsc_asiocr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ASIOCR));
		dev->vaddr.gsc_bcfgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BCFGR));
		dev->vaddr.gsc_bctlr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BCTLR));
		dev->vaddr.gsc_bstsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BSTSR));
		dev->vaddr.gsc_bufsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BUFSR));
		dev->vaddr.gsc_ibcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_IBCR));
		dev->vaddr.gsc_icr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ICR));
		dev->vaddr.gsc_idbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_IDBR));
		dev->vaddr.gsc_llcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_LLCR));
		dev->vaddr.gsc_ragr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_RAGR));
		dev->vaddr.gsc_rbgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_RBGR));
		dev->vaddr.gsc_smlwr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_SMLWR));
		dev->vaddr.gsc_smuwr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_SMUWR));
		dev->vaddr.gsc_sscr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_SSCR));

		dev->vaddr.gsc_ttacmr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_TTACMR));
		dev->vaddr.gsc_ttbsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_TTBSR));
		dev->vaddr.gsc_ttclr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_TTCLR));
		dev->vaddr.gsc_ttcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_TTCR));
		dev->vaddr.gsc_ttcrmr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_TTCRMR));
		dev->vaddr.gsc_ttcur_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_TTCUR));
		dev->vaddr.gsc_ttrdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_TTRDR));

		dev->vaddr.gsc_ll_data_32	= GSC_VADDR(dev, 0x100);

		// Data cache initialization.
		dev->cache.gsc_bcfgr_32			= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcfgr_32);

		dev->cache.autocal_ms			= 2000;
		dev->cache.fifo_size			= _256K;
		dev->cache.fsamp_max			= 1000000L;
		dev->cache.fsamp_min			= 1L;
		dev->cache.fw_ver				= dev->cache.gsc_bcfgr_32 & 0xFFF;
		dev->cache.initialize_ms		= 3;
		dev->cache.rate_gen_fgen_max	= 1000000L;
		dev->cache.rate_gen_fgen_min	= 1;
		dev->cache.rate_gen_nrate_max	= 0xFFFF;
		dev->cache.rate_gen_nrate_min	= 64;
		dev->cache.rate_gen_qty			= 2;

		ret	= _channels_compute(dev);
		if (ret)	break;

		ret	= _master_clock_compute(dev);
		if (ret)	break;

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		dma	= GSC_DMA_SEL_STATIC
			| GSC_DMA_CAP_BMDMA_READ
			| GSC_DMA_CAP_DMDMA_READ;
		ret	= gsc_dma_create(dev, dma, dma);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


