// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSC1M/16AI32SSC1M_Linux_1.x.x.x_DN/driver/main.h $
// $Rev: 53825 $
// $Date: 2023-10-04 16:30:11 -0500 (Wed, 04 Oct 2023) $

// 16AI32SSC1M: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "16ai32ssc1m.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"16AI32SSC1M"	// Upper case form of the below.
#define	DEV_NAME							"16ai32ssc1m"	// MUST AGREE WITH AI32SSC1M_BASE_NAME

#define	DEV_VERSION							"1.3"		// FOR DEVICE SPECIFIC CODE ONLY!
// 1.3	Standardized various IOCTL system log messages.
//		Standardized processing of the Initialize IOCTL service.
//		Standardized processing of the Autocalibrate IOCTL service.
//		Standardized processing of the Input Buffer Clear IOCTL service.
//		Removed compiler warnings under the 5.14 kernel.
//		Updated comments for consistency.
//		Updated "start" script to prevent accidentail deletion of device nodes with near identical names.
//		Updated to support segregation of PLX specific sources.
//		Updated to support the 6.x series kernel.
//		Renamed all Auto_Calibration content to Autocal.
//		Renamed all Auto_Cal_Status content to Autocal_Status.
// 1.2	Added support for the 5.x kernel series.
// 1.1	Updated per changes to common I/O code.
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AI32SSC1M_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AI32SSC1M_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AI32SSC1M_WAIT_IO_ALL


// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					dma_threshold;		// DMA should be this size or greater.
	s32					io_mode;			// PIO, DMA, DMDMA
	s32					overflow_check;		// Check overflow when reading?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check underflow when reading?

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of board's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of board's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_sem_t			sem;				// Only one Tx or Rx at a time.
	os_mem_t			mem;				// I/O buffer.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic board type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the board being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;			// Analog Input read
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bctlr_32;	// 0x00 Board Control Register
		VADDR_T			gsc_icr_32;		// 0x04 Interrupt Control Register
		VADDR_T			gsc_idbr_32;	// 0x08 Input Data Buffer Register
		VADDR_T			gsc_ibcr_32;	// 0x0C Input Buffer Control Register
		VADDR_T			gsc_ragr_32;	// 0x10 Rate-A Generator Register
		VADDR_T			gsc_rbgr_32;	// 0x14 Rate-B Generator Register
		VADDR_T			gsc_bufsr_32;	// 0x18 Buffer Size Register
		VADDR_T			gsc_bstsr_32;	// 0x1C Burst Size Register
		VADDR_T			gsc_sscr_32;	// 0x20 Scan & Sync Control Register
		VADDR_T			gsc_acar_32;	// 0x24 Active Channel Assignment Register
		VADDR_T			gsc_bcfgr_32;	// 0x28 Board Configuration Register

		VADDR_T			gsc_asiocr_32;	// 0x34 Auxiliary Sync I/O Control Register
		VADDR_T			gsc_smlwr_32;	// 0x38 Scan Marker Lower Word Register
		VADDR_T			gsc_smuwr_32;	// 0x3C Scan Marker Upper Word Register
		VADDR_T			gsc_llcr_32;	// 0x40 Low Latency Control Register
		VADDR_T			gsc_ll_data_32;	// 0x100 Low Latency Data Registers Base

		// These are the Time Tag registers

		VADDR_T			gsc_ttcr_32;	// 0x50 Time Tag Configuration Register
		VADDR_T			gsc_ttacmr_32;	// 0x54 Time Tag Active Channel Mask Register
		VADDR_T			gsc_ttclr_32;	// 0x58 Time Tag Counter Lower Register
		VADDR_T			gsc_ttcur_32;	// 0x5C Time Tag Counter Upper Register
		VADDR_T			gsc_ttrdr_32;	// 0x60 Time Tag Rate Divider Register
		VADDR_T			gsc_ttbsr_32;	// 0x64 Time Tag Burst Size Register
		VADDR_T			gsc_ttcrmr_32;	// 0x68 Time Tag Constant Reference Mask Register

	} vaddr;

	struct
	{
		u32				gsc_bcfgr_32;		// Board Configuration Register
		u32				fw_ver;

		s32				autocal_ms;			// Maximum ms for autocalibrate

		s32				channels_max;		// Maximum channels supported by model.
		s32				channel_qty;		// The number of channels on the board.

		u32				fifo_size;			// Size of FIFO - not the fill level.
		s32				fsamp_max;			// The maximum Fsamp rate per channel.
		s32				fsamp_min;			// The minimum Fsamp rate per channel.

		s32				initialize_ms;		// Maximum ms for initialize

		s32				master_clock;		// Master clock frequency

		s32				rate_gen_fgen_max;	// Rate Generator maximum output rate.
		s32				rate_gen_fgen_min;	// Rate Generator minimum output rate.
		s32				rate_gen_nrate_max;	// Minimum Nrate value.
		s32				rate_gen_nrate_min;	// Maximum Nrate value.
		s32				rate_gen_qty;		// The number of rate generators on board.

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
