// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSC1M/16AI32SSC1M_Linux_1.x.x.x_DN/samples/wait/perform.c $
// $Rev: 53821 $
// $Date: 2023-10-04 16:24:27 -0500 (Wed, 04 Oct 2023) $

// 16AI32SSC1M: Sample Application: source file

#include "main.h"



// data types *****************************************************************

typedef struct
{
	const char*	title;
	u32			main;
	u32			gsc;
	u32			io;
	int			(*callback)(int fd);
} options_t;



// variables ******************************************************************

static const options_t options[]	=
{
	// title					main								gsc		io	callback
	{ "PCI",					GSC_WAIT_MAIN_PCI,					0,		0,	wait_main_pci_test					},
	{ "DMA0/DMA1",				GSC_WAIT_MAIN_DMA0 |
								GSC_WAIT_MAIN_DMA1,					0,		0,	wait_main_dma_test					},
	{ "GSC",					GSC_WAIT_MAIN_GSC,					0,		0,	wait_main_gsc_test					},

	// title					main	gsc									io	callback
	{ "IRQ0 Init Done",			0,		AI32SSC1M_WAIT_GSC_INIT_DONE,		0,	wait_gsc_irq0_initialize_test		},
	{ "IRQ0 Autocal Done",		0,		AI32SSC1M_WAIT_GSC_AUTOCAL_DONE,	0,	wait_gsc_irq0_autocal_test			},
	{ "IRQ0 Sync Start",		0,		AI32SSC1M_WAIT_GSC_SYNC_START,		0,	wait_gsc_irq0_sync_start_test		},
	{ "IRQ0 Sync Done",			0,		AI32SSC1M_WAIT_GSC_SYNC_DONE,		0,	wait_gsc_irq0_sync_done_test		},
	{ "IRQ0 Burst Start",		0,		AI32SSC1M_WAIT_GSC_BURST_START,		0,	wait_gsc_irq0_burst_start_test		},
	{ "IRQ0 Burst Done",		0,		AI32SSC1M_WAIT_GSC_BURST_DONE,		0,	wait_gsc_irq0_burst_done_test		},
	{ "IRQ1 Buf Thresh L->H",	0,		AI32SSC1M_WAIT_GSC_IN_BUF_THR_L2H,	0,	wait_gsc_irq1_in_buf_thr_l2h_test	},
	{ "IRQ1 Buf Thresh H->L",	0,		AI32SSC1M_WAIT_GSC_IN_BUF_THR_H2L,	0,	wait_gsc_irq1_in_buf_thr_h2l_test	},
	{ "IRQ1 Error (Over)",		0,		AI32SSC1M_WAIT_GSC_IN_BUF_OVR_UNDR,	0,	wait_gsc_irq1_in_buf_overflow_test	},
	{ "IRQ1 Error (Under)",		0,		AI32SSC1M_WAIT_GSC_IN_BUF_OVR_UNDR,	0,	wait_gsc_irq1_in_buf_ubderflow_test	},

	// title					main	gsc	io									callback
	{ "I/O Rx Abort",			0,		0,	AI32SSC1M_WAIT_IO_RX_ABORT,			wait_io_rx_abort_test				},
	{ "I/O Rx Done",			0,		0,	AI32SSC1M_WAIT_IO_RX_DONE,			wait_io_rx_done_test				},
	{ "I/O Rx Error",			0,		0,	AI32SSC1M_WAIT_IO_RX_ERROR,			wait_io_rx_error_test				},
	{ "I/O Rx Timeout",			0,		0,	AI32SSC1M_WAIT_IO_RX_TIMEOUT,		wait_io_rx_timeout_test				},
	{ NULL }
};



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	int	i;

	for (i = 0; options[i].title; i++)
	{
		gsc_label(options[i].title);
		errs	+= service_test(args->fd, options[i].main, options[i].gsc, options[i].io, options[i].callback);
	}

	return(errs);
}


