// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSC1M/16AI32SSC1M_Linux_1.x.x.x_DN/utils/config_ai.c $
// $Rev: 53814 $
// $Date: 2023-10-04 16:21:01 -0500 (Wed, 04 Oct 2023) $

// 16AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_config_ai
*
*	Purpose:
*
*		Configure the device for AI operation.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the board to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ai32ssc1m_config_ai(int fd, int index, int verbose, s32 fsamp)
{
	s32		enable_a;
	s32		enable_b;
	int		errs	= 0;
	s32		nrate_a;
	s32		nrate_b;
	s32		qty		= 32;
	double	rate;
	s32		src;
	s32		src_b;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ai32ssc1m_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= ai32ssc1m_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= ai32ssc1m_rx_io_overflow		(fd, index, verbose, AI32SSC1M_IO_OVERFLOW_CHECK,		NULL);
	errs	+= ai32ssc1m_rx_io_timeout		(fd, index, verbose, AI32SSC1M_IO_TIMEOUT_DEFAULT,		NULL);
	errs	+= ai32ssc1m_rx_io_underflow	(fd, index, verbose, AI32SSC1M_IO_UNDERFLOW_CHECK,		NULL);
	errs	+= ai32ssc1m_ai_buf_thr_lvl		(fd, index, verbose, 250000,							NULL);
	errs	+= ai32ssc1m_ai_mode			(fd, index, verbose, AI32SSC1M_AI_MODE_DIFF,			NULL);
	errs	+= ai32ssc1m_ai_range			(fd, index, verbose, AI32SSC1M_AI_RANGE_10V,			NULL);
	errs	+= ai32ssc1m_burst_size			(fd, index, verbose, 1,									NULL);
	errs	+= ai32ssc1m_burst_sync			(fd, index, verbose, AI32SSC1M_BURST_SYNC_DISABLE,		NULL);
	errs	+= ai32ssc1m_chan_active		(fd, index, verbose, AI32SSC1M_CHAN_ACTIVE_RANGE,		NULL);
	errs	+= ai32ssc1m_chan_first			(fd, index, verbose, 0,									NULL);

	errs	+= ai32ssc1m_query				(fd, index, verbose, AI32SSC1M_QUERY_CHANNEL_QTY, &qty);
	errs	+= ai32ssc1m_chan_last			(fd, index, verbose, qty - 1,							NULL);

	errs	+= ai32ssc1m_chan_single		(fd, index, verbose, 0,									NULL);
	errs	+= ai32ssc1m_data_format		(fd, index, verbose, AI32SSC1M_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= ai32ssc1m_data_packing		(fd, index, verbose, AI32SSC1M_DATA_PACKING_DISABLE,	NULL);
	errs	+= ai32ssc1m_io_inv				(fd, index, verbose, AI32SSC1M_IO_INV_HIGH,				NULL);
	errs	+= ai32ssc1m_scan_marker_set	(fd, index, verbose, 0);
	errs	+= ai32ssc1m_fsamp_ai_compute	(fd, index, verbose, fsamp, &src, &src_b, &nrate_a, &nrate_b, &enable_a, &enable_b, &rate);
	errs	+= ai32ssc1m_adc_clk_src		(fd, index, verbose, src,								NULL);
	errs	+= ai32ssc1m_rag_nrate			(fd, index, verbose, nrate_a,							NULL);
	errs	+= ai32ssc1m_rag_enable			(fd, index, verbose, enable_a,							NULL);
	errs	+= ai32ssc1m_rbg_clk_src		(fd, index, verbose, src_b,								NULL);
	errs	+= ai32ssc1m_rbg_nrate			(fd, index, verbose, nrate_b,							NULL);
	errs	+= ai32ssc1m_rbg_enable			(fd, index, verbose, enable_b,							NULL);
	errs	+= ai32ssc1m_rbg_sync_output	(fd, index, verbose, AI32SSC1M_RBG_SYNC_OUTPUT_DISABLE,	NULL);
	errs	+= ai32ssc1m_scan_marker		(fd, index, verbose, AI32SSC1M_SCAN_MARKER_DISABLE,		NULL);
	errs	+= ai32ssc1m_adc_enable			(fd, index, verbose, AI32SSC1M_ADC_ENABLE_YES,			NULL);

	errs	+= ai32ssc1m_fsamp_ai_report_all(fd, index, verbose,									NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the board user manual for clarification.
	errs	+= ai32ssc1m_autocal			(fd, index, verbose);

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= ai32ssc1m_irq0_sel			(fd, index, verbose, AI32SSC1M_IRQ0_INIT_DONE,			NULL);
	errs	+= ai32ssc1m_irq1_sel			(fd, index, verbose, AI32SSC1M_IRQ1_NONE,				NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the overflow and underflow status bits.
	errs	+= ai32ssc1m_ai_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


