// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSC1M/16AI32SSC1M_Linux_1.x.x.x_DN/utils/fsamp_ai.c $
// $Rev: 53814 $
// $Date: 2023-10-04 16:21:01 -0500 (Wed, 04 Oct 2023) $

// 16AI32SSC1M: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))



// data types *****************************************************************

typedef struct
{
	s32			fsamp_want;
	long double	fsamp_got;

	long double	delta;
	long double	error;		// absolute of delta
	s32			enable_a;
	s32			enable_b;
	s32			master;
	s32			fsamp_max;
	s32			fsamp_min;
	s32			nrate_a;
	s32			nrate_b;
	s32			nrate_max;
	s32			nrate_min;
	s32			src;
	s32			src_b;
} compute_t;

typedef struct
{
	char		buf[1024];

	s32			adc_enable;
	s32			burst;
	s32			chan_active;
	s32			chan_first;
	s32			chan_last;
	u32			chan_mask;
	s32			chan_max;
	s32			chan_qty;
	s32			chan_single;
	s32			clk_src;
	s32			fref;
	s32			fsamp;
	s32			rag_enable;
	s32			rag_nrate;
	s32			rbg_enable;
	s32			rbg_nrate;
	s32			rbg_clk_src;

	struct
	{
		s32		adc_enable;
		u32		chan_mask;
		s32		clk_src;
		s32		enable;
		s32		nrate;
	} tt;
} report_t;



//*****************************************************************************
static int _get_active_channels(int fd, report_t* report)
{
	int	errs	= 0;

	if (report->tt.enable)
	{
		report->chan_mask	= 0;
	}
	else
	{
		switch (report->chan_active)
		{
			default:

				errs++;
				report->chan_mask	= 0;
				sprintf(report->buf,
						"Invalid CHAN_ACTIVE option: %ld",
						(long) report->chan_active);
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_1:		report->chan_mask	= 0x00000003;		break;
			case AI32SSC1M_CHAN_ACTIVE_0_3:		report->chan_mask	= 0x0000000F;		break;
			case AI32SSC1M_CHAN_ACTIVE_0_7:		report->chan_mask	= 0x000000FF;		break;
			case AI32SSC1M_CHAN_ACTIVE_0_15:	report->chan_mask	= 0x0000FFFF;		break;
			case AI32SSC1M_CHAN_ACTIVE_0_31:	report->chan_mask	= 0xFFFFFFFF;		break;
			case AI32SSC1M_CHAN_ACTIVE_SINGLE:

				report->chan_mask	= 0x1L << report->chan_single;
				break;

			case AI32SSC1M_CHAN_ACTIVE_RANGE:

				report->chan_mask	= GSC_FIELD_ENCODE(0xFFFFFFFF, report->chan_last, report->chan_first);
				break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _report_rag(int fd, report_t* report)
{
	int	errs	= 0;

	if (report->rag_enable == AI32SSC1M_GEN_ENABLE_NO)
	{
		report->fsamp	= 0;
		strcpy(report->buf, "RAG Disabled");
	}
	else if (report->rag_nrate == 0)
	{
		errs++;
		report->fsamp	= 0;
		strcpy(report->buf, "RAG Nrate is zero");
	}
	else
	{
		report->fsamp	= (s32) (((float) report->fref) / report->rag_nrate);
		sprintf(report->buf,
				"Fref %ld, RAG Nrate %ld",
				(long) report->fref,
				(long) report->rag_nrate);
	}

	return(errs);
}



//*****************************************************************************
static int _report_rbg_master(int fd, report_t* report)
{
	int	errs	= 0;

	if (report->rbg_enable == AI32SSC1M_GEN_ENABLE_NO)
	{
		report->fsamp	= 0;
		strcpy(report->buf, "RBG Disabled");
	}
	else if (report->rbg_nrate == 0)
	{
		errs++;
		report->fsamp	= 0;
		strcpy(report->buf, "RBG Nrate is zero");
	}
	else
	{
		report->fsamp	= (s32) (((float) report->fref) / report->rbg_nrate);
		sprintf(report->buf,
				"Fref %ld, RBG Nrate %ld",
				(long) report->fref,
				(long) report->rbg_nrate);
	}

	return(errs);
}



//*****************************************************************************
static int _report_rbg_rag(int fd, report_t* report)
{
	int	errs	= 0;

	if (report->rag_enable == AI32SSC1M_GEN_ENABLE_NO)
	{
		report->fsamp	= 0;
		strcpy(report->buf, "RAG Disabled");
	}
	else if (report->rbg_enable == AI32SSC1M_GEN_ENABLE_NO)
	{
		report->fsamp	= 0;
		strcpy(report->buf, "RBG Disabled");
	}
	else if (report->rag_nrate == 0)
	{
		errs++;
		report->fsamp	= 0;
		strcpy(report->buf, "RAG Nrate is zero");
	}
	else if (report->rbg_nrate == 0)
	{
		errs++;
		report->fsamp	= 0;
		strcpy(report->buf, "RBG Nrate is zero");
	}
	else
	{
		report->fsamp	= (s32) (((float) report->fref) / report->rag_nrate / report->rbg_nrate);
		sprintf(report->buf,
				"Fref %ld, RAG Nrate %ld, RBG Nrate %ld",
				(long) report->fref,
				(long) report->rag_nrate,
				(long) report->rbg_nrate);
	}

	return(errs);
}



//*****************************************************************************
static int _report_rbg(int fd, report_t* report)
{
	int	errs	= 0;

	switch (report->rbg_clk_src)
	{
		default:

			errs++;
			report->fsamp	= 0;
			sprintf(report->buf,
					"Invalid RBG CLK SRC option: %ld",
					(long) report->rbg_clk_src);
			break;


		case AI32SSC1M_RBG_CLK_SRC_MASTER:

			errs	+= _report_rbg_master(fd, report);
			break;

		case AI32SSC1M_RBG_CLK_SRC_RAG:

			errs	+= _report_rbg_rag(fd, report);
			break;
	}

	return(errs);
}



//*****************************************************************************
static int _report_tt_rag(int fd, report_t* report)
{
	int	errs	= 0;

	if (report->rag_enable == AI32SSC1M_GEN_ENABLE_NO)
	{
		report->fsamp	= 0;
		strcpy(report->buf, "RAG Disabled");
	}
	else if (report->rag_nrate == 0)
	{
		errs	= 1;
		report->fsamp	= 0;
		strcpy(report->buf, "RAG Nrate is zero");
	}
	else if (report->tt.nrate == 0)
	{
		errs	= 1;
		report->fsamp	= 0;
		strcpy(report->buf, "TT Nrate is zero");
	}
	else
	{
		report->fsamp	= (s32) (((float) report->fref) / report->rag_nrate / report->tt.nrate);
		sprintf(report->buf,
				"Fref %ld, RAG Nrate %ld, TT Nrate %ld",
				(long) report->fref,
				(long) report->rag_nrate,
				(long) report->tt.nrate);
	}

	return(errs);
}



//*****************************************************************************
static int _report_tt(int fd, report_t* report)
{
	int	errs	= 0;;

	if (report->tt.adc_enable == AI32SSC1M_TT_ADC_ENABLE_NO)
	{
		report->fsamp	= 0;
		strcpy(report->buf, "TT ADC Disabled");
	}
	else if (report->tt.clk_src == AI32SSC1M_TT_ADC_CLK_SRC_RAG)
	{
		errs	= _report_tt_rag(fd, report);
	}
	else
	{
		report->fsamp	= 0;
		strcpy(report->buf, "TT External Clocking");
	}

	return(errs);
}



//*****************************************************************************
static void _fsamp_compute_a(compute_t* data)
{
	long double	delta;
	long double	error;
	long double	fsamp_got;
	long double	fsamp_want	= data->fsamp_want;
	int			init		= 0;	// Have we performed the first calculation?
	s32			master		= data->master;
	s32			nr;
	s32			nr_max;
	s32			nr_min;
	s32			nrate_a;
	s32			nrate_max	= data->nrate_max;
	s32			nrate_min	= data->nrate_min;

	data->src		= AI32SSC1M_ADC_CLK_SRC_RAG;
	data->src_b		= AI32SSC1M_RBG_CLK_SRC_MASTER;
	data->enable_a	= AI32SSC1M_GEN_ENABLE_YES;
	data->enable_b	= AI32SSC1M_GEN_ENABLE_NO;

	nr		= master / fsamp_want;
	nr		= (nr < nrate_min) ? nrate_min
			: (nr > nrate_max) ? nrate_max : nr;

	nr_min	= nr - 5;
	nr_max	= nr + 5;
	nr_min	= (nr_min < nrate_min) ? nrate_min
			: (nr_min > nrate_max) ? nrate_max : nr_min;
	nr_max	= (nr_max < nrate_min) ? nrate_min
			: (nr_max > nrate_max) ? nrate_max : nr_max;

	for (nrate_a = nr_min; nrate_a <= nr_max; nrate_a++)
	{
		fsamp_got	= (long double) master / nrate_a;
		delta		= fsamp_got - fsamp_want;
		error		= ABS(delta);

		if ((init == 0) || (error < data->error))
		{
			init			= 1;
			data->delta		= delta;
			data->error		= error;
			data->fsamp_got	= fsamp_got;
			data->nrate_a	= nrate_a;
		}
	}
}



//*****************************************************************************
static void _fsamp_compute_ab(compute_t* data)
{
	long double	delta;
	long double	error;
	long double	fsamp_got;
	long double	fsamp_want	= data->fsamp_want;
	int			init		= 0;	// Have we performed the first calculation?
	s32			master		= data->master;
	s32			nr;
	s32			nr_max;
	s32			nr_min;
	s32			nrate_a;
	s32			nrate_b;
	s32			nrate_max	= data->nrate_max;
	s32			nrate_min	= data->nrate_min;

	data->src		= AI32SSC1M_ADC_CLK_SRC_RBG;
	data->src_b		= AI32SSC1M_RBG_CLK_SRC_RAG;
	data->enable_a	= AI32SSC1M_GEN_ENABLE_YES;
	data->enable_b	= AI32SSC1M_GEN_ENABLE_YES;

	for (nrate_a = nrate_min; nrate_a <= nrate_max; nrate_a++)
	{
		nr		= master / fsamp_want / nrate_a;
		nr		= (nr < nrate_min) ? nrate_min
				: (nr > nrate_max) ? nrate_max : nr;

		nr_min	= nr - 5;
		nr_max	= nr + 5;
		nr_min	= (nr_min < nrate_min) ? nrate_min
				: (nr_min > nrate_max) ? nrate_max : nr_min;
		nr_max	= (nr_max < nrate_min) ? nrate_min
				: (nr_max > nrate_max) ? nrate_max : nr_max;

		for (nrate_b = nr_min; nrate_b <= nr_max; nrate_b++)
		{
			fsamp_got	= (long double) master / ((long long) nrate_b * nrate_a);
			delta		= fsamp_got - fsamp_want;
			error		= ABS(delta);

			if ((init == 0) || (error < data->error))
			{
				init			= 1;
				data->delta		= delta;
				data->error		= error;
				data->fsamp_got	= fsamp_got;
				data->nrate_a	= nrate_a;
				data->nrate_b	= nrate_b;
			}
		}
	}
}



//*****************************************************************************
int	ai32ssc1m_fsamp_ai_compute(
	int		fd,
	int		index,
	int		verbose,
	s32		fsamp,
	s32*	src,
	s32*	src_b,
	s32*	nrate_a,
	s32*	nrate_b,
	s32*	enable_a,
	s32*	enable_b,
	double*	rate)
{
	compute_t	data;
	compute_t	data_a;
	compute_t	data_ab;
	int			errs		= 0;

	if (verbose)
		gsc_label_index("Fsamp AI Compute", index);

	memset(&data, 0, sizeof(data));
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX, &data.fsamp_max);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MIN, &data.fsamp_min);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK, &data.master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_NRATE_MAX, &data.nrate_max);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_NRATE_MIN, &data.nrate_min);

	data.error		= LDBL_MAX;
	data.fsamp_want	= fsamp;

	data_a	= data;
	data_ab	= data;

	_fsamp_compute_a(&data_a);
	_fsamp_compute_ab(&data_ab);

	if (data_a.error < data_ab.error)
	{
		data_a.nrate_b	= data.nrate_max;
		data			= data_a;

		if (verbose)
		{
			printf("%s  (", errs ? "FAIL <---" : "PASS");
			gsc_label_long_comma(fsamp);
			printf(" S/S: ");
			printf("Fref ");
			gsc_label_long_comma((long) data.master);
			printf(", Nrate-A ");
			gsc_label_long_comma((long) data.nrate_a);
			printf(", Fsamp %.3Lf", data.fsamp_got);
			printf(")\n");
		}
	}
	else
	{
		data	= data_ab;

		if (verbose)
		{
			printf("%s  (", errs ? "FAIL <---" : "PASS");
			gsc_label_long_comma(fsamp);
			printf(" S/S: ");
			printf("Fref ");
			gsc_label_long_comma((long) data.master);
			printf(", Nrate-A ");
			gsc_label_long_comma((long) data.nrate_a);
			printf(", Nrate-B ");
			gsc_label_long_comma((long) data.nrate_b);
			printf(", Fsamp %.3Lf", data.fsamp_got);
			printf(")\n");
		}
	}

	if (src)
		src[0]		= data.src;

	if (src_b)
		src_b[0]	= data.src_b;

	if (nrate_a)
		nrate_a[0]	= data.nrate_a;

	if (nrate_b)
		nrate_b[0]	= data.nrate_b;

	if (enable_a)
		enable_a[0]	= data.enable_a;

	if (enable_b)
		enable_b[0]	= data.enable_b;

	if (rate)
		rate[0]	= data.fsamp_got;

	return(errs);
}



//*****************************************************************************
int ai32ssc1m_fsamp_ai_report(int fd, int index, int verbose, int chan, s32* rate)
{
	u32			bit;
	int			errs		= 0;
	report_t	report;

	if (verbose)
	{
		sprintf(report.buf, "Channel %d", chan);
		gsc_label_index(report.buf, index);
	}

	for (;;)
	{
		memset(&report, 0, sizeof(report));

		errs	+= ai32ssc1m_query				(fd, -1, 0, AI32SSC1M_QUERY_CHANNEL_MAX, &report.chan_max);
		errs	+= ai32ssc1m_query				(fd, -1, 0, AI32SSC1M_QUERY_CHANNEL_QTY, &report.chan_qty);
		errs	+= ai32ssc1m_query				(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK, &report.fref);

		errs	+= ai32ssc1m_adc_clk_src		(fd, -1, 0, -1, &report.clk_src);

		errs	+= ai32ssc1m_adc_enable			(fd, -1, 0, -1, &report.adc_enable);

		errs	+= ai32ssc1m_burst_sync			(fd, -1, 0, -1, &report.burst);
		errs	+= ai32ssc1m_chan_active		(fd, -1, 0, -1, &report.chan_active);
		errs	+= ai32ssc1m_chan_first			(fd, -1, 0, -1, &report.chan_first);
		errs	+= ai32ssc1m_chan_last			(fd, -1, 0, -1, &report.chan_last);
		errs	+= ai32ssc1m_chan_single		(fd, -1, 0, -1, &report.chan_single);
		errs	+= ai32ssc1m_rag_enable			(fd, -1, 0, -1, &report.rag_enable);
		errs	+= ai32ssc1m_rag_nrate			(fd, -1, 0, -1, &report.rag_nrate);

		errs	+= ai32ssc1m_rbg_enable			(fd, -1, 0, -1, &report.rbg_enable);
		errs	+= ai32ssc1m_rbg_nrate			(fd, -1, 0, -1, &report.rbg_nrate);
		errs	+= ai32ssc1m_rbg_clk_src		(fd, -1, 0, -1, &report.rbg_clk_src);

		errs	+= ai32ssc1m_tt_adc_enable		(fd, -1, 0, -1, &report.tt.adc_enable);
		errs	+= ai32ssc1m_tt_chan_mask_get	(fd, -1, 0, &report.tt.chan_mask);
		errs	+= ai32ssc1m_tt_adc_clk_src		(fd, -1, 0, -1, &report.tt.clk_src);
		errs	+= ai32ssc1m_tt_enable			(fd, -1, 0, -1, &report.tt.enable);
		errs	+= ai32ssc1m_tt_nrate			(fd, -1, 0, -1, &report.tt.nrate);

		errs	+= _get_active_channels(fd, &report);

		if (errs)
		{
			report.fsamp	= 0;
			break;
		}

		if ((chan < 0) || (chan >= report.chan_max))
		{
			errs++;
			report.fsamp	= 0;
			sprintf(report.buf, "Invalid channel index: %d", chan);
			break;
		}

		if (chan >= report.chan_qty)
		{
			report.fsamp	= 0;
			sprintf(report.buf, "Channel not installed.");
			break;
		}

		if (report.tt.enable)
		{
			errs	+= _report_tt(fd, &report);
			break;
		}

		if (report.adc_enable == 0)
		{
			report.fsamp	= 0;
			sprintf(report.buf, "ADC disabled.");
			break;
		}

		if (report.burst)
		{
			report.fsamp	= 0;
			sprintf(report.buf, "Bursting enabled.");
			break;
		}

		bit	= 1 << chan;

		if ((bit & report.chan_mask) == 0)
		{
			report.fsamp	= 0;
			sprintf(report.buf, "Channel disabled.");
			break;
		}

		switch (report.clk_src)
		{
			default:

				errs++;
				report.fsamp	= 0;
				sprintf(report.buf, "Invalid clock source: %ld", (long) report.clk_src);
				break;

			case AI32SSC1M_ADC_CLK_SRC_EXT:

				report.fsamp	= 0;
				sprintf(report.buf, "External clocking.");
				break;

			case AI32SSC1M_ADC_CLK_SRC_RAG:

				errs	+= _report_rag(fd, &report);
				break;

			case AI32SSC1M_ADC_CLK_SRC_RBG:

				errs	+= _report_rbg(fd, &report);
				break;

			case AI32SSC1M_ADC_CLK_SRC_BCR:

				report.fsamp	= 0;
				sprintf(report.buf, "Software clocking.");
				break;
		}

		break;
	}

	if (verbose)
	{
		gsc_label_long_comma((long) report.fsamp);
		printf(" S/S");

		if (errs)
			printf(" FAIL <---");

		if (report.buf[0])
			printf("  (%s)", report.buf);

		printf("\n");
	}

	if (rate)
		rate[0]	= report.fsamp;

	return(errs);
}



//*****************************************************************************
int ai32ssc1m_fsamp_ai_report_all(int fd, int index, int verbose, s32* rate)
{
	int	chan;
	int	errs;
	s32	qty;
	s32	sps;
	s32	total	= 0;

	if (verbose)
		gsc_label_index("Sample Rates", index);

	errs	= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_CHANNEL_QTY, &qty);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---\n");
	}
	else
	{
		if (verbose)
			printf("\n");

		gsc_label_level_inc();

		for (chan = 0; chan < qty; chan++)
		{
			errs	+= ai32ssc1m_fsamp_ai_report(fd, index, verbose, chan, &sps);
			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
		}

		gsc_label_level_dec();
	}

	if (rate)
		rate[0]	= total;

	return(errs);
}


