// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSC1M/16AI32SSC1M_Linux_1.x.x.x_DN/utils/query.c $
// $Rev: 53814 $
// $Date: 2023-10-04 16:21:01 -0500 (Wed, 04 Oct 2023) $

// 16AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSC1M_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai32ssc1m_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AI32SSC1M_QUERY_AUTOCAL_MS:		ptr	= "Autocal Period";		break;
		case AI32SSC1M_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
		case AI32SSC1M_QUERY_CHANNEL_QTY:		ptr	= "Input Channels";		break;
		case AI32SSC1M_QUERY_COUNT:				ptr	= "Query Options";		break;
		case AI32SSC1M_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
		case AI32SSC1M_QUERY_FGEN_MAX:			ptr	= "Fgen Max";			break;
		case AI32SSC1M_QUERY_FGEN_MIN:			ptr	= "Fgen Min";			break;
		case AI32SSC1M_QUERY_FIFO_SIZE:			ptr	= "FIFO Size";			break;
		case AI32SSC1M_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";			break;
		case AI32SSC1M_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";			break;
		case AI32SSC1M_QUERY_INIT_MS:			ptr	= "Initialize Period";	break;
		case AI32SSC1M_QUERY_MASTER_CLOCK:		ptr	= "Master Clock";		break;
		case AI32SSC1M_QUERY_NRATE_MAX:			ptr = "Nrate Max";			break;
		case AI32SSC1M_QUERY_NRATE_MIN:			ptr = "Nrate Min";			break;
		case AI32SSC1M_QUERY_RATE_GEN_QTY:		ptr = "Rate Generators";	break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AI32SSC1M_QUERY_CHANNEL_MAX:
		case AI32SSC1M_QUERY_CHANNEL_QTY:
		case AI32SSC1M_QUERY_COUNT:
		case AI32SSC1M_QUERY_NRATE_MAX:
		case AI32SSC1M_QUERY_NRATE_MIN:
		case AI32SSC1M_QUERY_RATE_GEN_QTY:

			sprintf(buf, "%ld", (long) set);
			break;

		case AI32SSC1M_QUERY_FGEN_MAX:
		case AI32SSC1M_QUERY_FGEN_MIN:
		case AI32SSC1M_QUERY_MASTER_CLOCK:

			sprintf(buf, "%ld Hz", (long) set);
			break;

		case AI32SSC1M_QUERY_AUTOCAL_MS:
		case AI32SSC1M_QUERY_INIT_MS:

			sprintf(buf, "%ld ms", (long) set);
			break;

		case AI32SSC1M_QUERY_FSAMP_MAX:
		case AI32SSC1M_QUERY_FSAMP_MIN:

			sprintf(buf, "%ld S/S", (long) set);
			break;

		case AI32SSC1M_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			sprintf(buf, "%ld%s Words", (long) tmp, ptr);
			break;

		case AI32SSC1M_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16AI32SSC1M)
			{
				strcpy(buf, "16AI32SSC1M");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


