// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSC1M/16AI32SSC1M_Linux_1.x.x.x_DN/utils/reg.c $
// $Rev: 53814 $
// $Date: 2023-10-04 16:21:01 -0500 (Wed, 04 Oct 2023) $

// 16AI32SSC1M: Utilities: source file

#include "main.h"



#if 0
// For the file format script.
if (verbose
if (verbose
#endif



// macros *********************************************************************

#define	_GSC_REG(a)					"GSC " #a, AI32SSC1M_GSC_##a, 0, 0
#define	NAME_WIDTH					21
#define	SHOW_FIELD(b,e,l,p,n)		gsc_reg_field_show(width + 10, NAME_WIDTH, value, (b), (e), (l), (p), (n))



//*****************************************************************************
static int _bctlr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	aim[]	=
	{
		"Differential",
		"RESERVED",
		"Zero Test",
		"+Vref Test",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	range[]	=
	{
		"+-1.25 Volts",
		"+-2.5 Volts",
		"+-5 Volts",
		"+-10 Volts"
	};

	static const char*	enabled[]	= { "Enabled",			"Disabled"		};
	static const char*	disabled[]	= { "Disabled",			"Enabled"		};
	static const char*	idle[]		= { "Idle",				"Active"		};
	static const char*	no[]		= { "No",				"Yes"			};
	static const char*	twos[]		= { "Twos Compliment",	"Offset Binary"	};

	SHOW_FIELD( 2,  0, 1, aim,		"Input Mode"		);
	SHOW_FIELD( 3,  3, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 5,  4, 1, range,	"Range"				);
	SHOW_FIELD( 6,  6, 1, twos,		"Data Format"		);
	SHOW_FIELD( 9,  7, 1, NULL,		"Reserved"			);
	SHOW_FIELD(10, 10, 1, no,		"Data On Hold"		);
	SHOW_FIELD(11, 11, 1, enabled,	"Scan Marker"		);
	SHOW_FIELD(12, 12, 1, idle,		"Input Sync"		);
	SHOW_FIELD(13, 13, 1, idle,		"Autocal"			);
	SHOW_FIELD(14, 14, 1, no,		"Autocal Pass"		);
	SHOW_FIELD(15, 15, 1, idle,		"Initialize"		);
	SHOW_FIELD(16, 16, 1, no,		"Buffer Underflow"	);
	SHOW_FIELD(17, 17, 1, no,		"Buffer Overflow"	);
	SHOW_FIELD(18, 18, 1, disabled,	"Data Packing"		);
	SHOW_FIELD(19, 19, 1, NULL,		"Reserved"			);
	SHOW_FIELD(20, 20, 1, disabled,	"Time Tag"			);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _icr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	irq0[]	=
	{
		"Initialization Complete",
		"Autocal Completed",
		"Input Sample Initiated (Sync)",
		"Input Sample Completed (data ready)",
		"Trigger Burst Initiated (Burst Busy -> High",
		"Trigger Burst Completed (Burst Busy -> Low",
		"Reserved",
		"Reserved"
	};

	static const char*	irq1[]	=
	{
		"Idle/None",
		"Input Buffer Threshold Low->High",
		"Input Buffer Threshold High->Low",
		"Input Buffer Overflow/Underflow",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	negated[]	= { "Negated",	"Asserted"	};

	SHOW_FIELD( 2,  0, 1, irq0,		"IRQ0 Select"	);
	SHOW_FIELD( 3,  3, 1, negated,	"IRQ0 Request"	);
	SHOW_FIELD( 6,  4, 1, irq1,		"IRQ1 Select"	);
	SHOW_FIELD( 7,  7, 1, negated,	"IRQ1 Request"	);
	SHOW_FIELD(31,  8, 1, NULL,		"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _idbr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	tag[]	= { "Not First",	"First"	};

	SHOW_FIELD(15,  0, 1, NULL,	"Data"			);
	SHOW_FIELD(30, 16, 1, NULL,	"Reserved"		);
	SHOW_FIELD(31, 31, 1, tag,	"Channel Tag"	);
	return(0);
}



//*****************************************************************************
static int _ibcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]		= { "Idle",		"Active"		};
	static const char*	negate[]	= { "Negated",	"Asserted"		};

	SHOW_FIELD(17,  0, 1, NULL,		"Threshold"			);
	SHOW_FIELD(18, 18, 1, idle,		"Clear Buffer"		);
	SHOW_FIELD(19, 19, 1, negate,	"Threshold Flag"	);
	SHOW_FIELD(31, 20, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _rgr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	enabled[]	= { "Enabled",	"Disabled"	};

	SHOW_FIELD(15,  0, 1, NULL,		"Nrate"		);
	SHOW_FIELD(16, 16, 1, enabled,	"Disable"	);
	SHOW_FIELD(31, 17, 1, NULL,		"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _bufsr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(18,  0, 1, NULL,	"Filled Locations"	);
	SHOW_FIELD(31, 19, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _bstsr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(18,  0, 1, NULL,	"Burst Size"	);
	SHOW_FIELD(31, 19, 1, NULL,	"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _sscr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	burst[]	=
	{
		"Disabled",
		"Rate-B Generator",
		"Ext Sync Input (Selects Sync TARGET Mode)",
		"Software (BCTLR Input Sync control bit)"
	};

	static const char*	channels[]	=
	{
		"Single Channel Mode",
		"2 Channels (0-1)",
		"4 Channels (0-3)",
		"8 Channels (0-7)",
		"16 Channels (0-15)",
		"32 Channels (0-31)",
		"RESERVED",
		"Channel Range (x-y)"
	};

	static const char*	clk_src[]	=
	{
		"External Clock Input(Selects Clock TARGET Mode)",
		"Rate-A Generator",
		"Rate-B Generator",
		"Software (BCTLR Input Sync Control bit, if bursting enabled)"
	};

	static const char*	b_src[]		= { "Master Clock",	"Rate-A Generator"	};
	static const char*	disabled[]	= { "Disabled",		"Enabled"			};
	static const char*	idle[]		= { "Idle",			"Active"			};
	static const char*	no[]		= { "No",			"Yes"				};

	SHOW_FIELD( 2,  0, 1, channels,	"Active Channel"		);
	SHOW_FIELD( 4,  3, 1, clk_src,	"Sample Clock Source"	);
	SHOW_FIELD( 5,  5, 1, disabled,	"Enable Clocking"		);
	SHOW_FIELD( 6,  6, 1, no,		"Rate-B Sync Output"	);
	SHOW_FIELD( 7,  7, 1, idle,		"Bursting"				);
	SHOW_FIELD( 9,  8, 1, burst,	"Bursting Source"		);
	SHOW_FIELD(10, 10, 1, b_src,	"Rate-B Gen. Source"	);
	SHOW_FIELD(11, 11, 1, no,		"Invert I/O"			);
	SHOW_FIELD(17, 12, 1, NULL,		"Single Channel"		);
	SHOW_FIELD(31, 18, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _acar_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD( 7,  0, 1, NULL,	"First Channel"	);
	SHOW_FIELD(15,  8, 1, NULL,	"Last Channel"	);
	SHOW_FIELD(31, 16, 1, NULL,	"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _bcfgr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	custom[]	=
	{
		"None",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	channels[]	=
	{
		"32 Channels",
		"16 Channels",
		"RESERVED",
		"RESERVED"
	};

	static const char*	master[]	=
	{
		"64 MHz",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	SHOW_FIELD(11,  0, 1, NULL,		"Firmware Rev"		);
	SHOW_FIELD(15, 12, 1, NULL,		"Reserved"			);
	SHOW_FIELD(17, 16, 1, channels,	"Channels"			);
	SHOW_FIELD(19, 18, 1, master,	"Master Clock"		);
	SHOW_FIELD(21, 20, 1, custom,	"Custom Feature"	);
	SHOW_FIELD(31, 22, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _asiocr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clock[]	=
	{
		"Inactive",
		"Active Input (Low-to-High Edge)",
		"Active Output (Positive Pulse)",
		"Reserved"
	};

	static const char*	high[]	= { "High Pulses",		"Low Pulses"		};
	static const char*	low[]	= { "Low-to-High Edge",	"High-to-Low Edge"	};
	static const char*	noise[]	= { "100ns-135ns",		"1.5us-2.0us"		};

	SHOW_FIELD( 1,  0, 1, clock,	"Aux Clock Mode"	);
	SHOW_FIELD( 3,  2, 1, clock,	"Aux Sync Mode"		);
	SHOW_FIELD( 7,  4, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 8,  8, 1, low,		"Input Polarity"	);
	SHOW_FIELD( 9,  9, 1, high,		"Output Polarity"	);
	SHOW_FIELD(10, 10, 1, noise,	"Noise Supression"	);
	SHOW_FIELD(31, 11, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _smwr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15,  0, 1, NULL,	"Marker"	);
	SHOW_FIELD(31, 16, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _llcr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD( 5,  0, 1, NULL,	"Hold Channel"		);
	SHOW_FIELD(11,  6, 1, NULL,	"Release Channel"	);
	SHOW_FIELD(31, 12, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _ttcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clock[]	=
	{
		"Rate-A Generator",
		"SAMP CLK INP",
		"REF CLK INP",
		"Reserved"
	};

	static const char*	disabled[]	= { "Disabled",				"Enabled"			};
	static const char*	idle[]		= { "Idle",					"Reset"				};
	static const char*	logging[]	= { "Triggerred Channels",	"Active Channels"	};
	static const char*	source[]	= { "Internal",				"REF CLK INP"		};
	static const char*	trigger[]	= { "Free Run",				"Reference-Trigger"	};

	SHOW_FIELD( 1,  0, 1, clock,	"Clocking Source"	);
	SHOW_FIELD( 2,  2, 1, disabled,	"ADC Clocking"		);
	SHOW_FIELD( 3,  3, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 4,  4, 1, trigger,	"Reference Trigger"	);
	SHOW_FIELD( 5,  5, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 6,  6, 1, logging,	"Global Trigger"	);
	SHOW_FIELD( 7,  7, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 8,  8, 1, source,	"Ext. Clock Source"	);
	SHOW_FIELD( 9,  9, 1, idle,		"Time Tag Reset"	);
	SHOW_FIELD(10, 10, 1, disabled,	"External Reset"	);
	SHOW_FIELD(11, 11, 1, disabled,	"Tagging"			);
	SHOW_FIELD(31, 12, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _ttacmr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	no[]	= { "No",	"Yes"	};

	char	buf[64];
	s32		chans	= 32;
	int		errs;
	int		i;

	errs	= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_CHANNEL_QTY, &chans);
	chans	= (chans < 1) ? 1 : ((chans > 32) ? 32 : chans);

	for (i = 0; i <= 31; i++)
	{
		if ((i + 1) > chans)
		{
			SHOW_FIELD(i, i, 1, NULL,	"Reserved"	);
		}
		else
		{
			sprintf(buf, "Acquire Chan %d", i);
			SHOW_FIELD(i, i, 1, no,	buf);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _ttclr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31, 0, 1, NULL, "Lower 32-bits");
	return(0);
}



//*****************************************************************************
static int _ttcur_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15,  0, 1, NULL, "Upper 16-bits"	);
	SHOW_FIELD(31, 16, 1, NULL, "Reserved"		);
	return(0);
}



//*****************************************************************************
static int _ttrdr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(19,  0, 1, NULL, "Divider"	);
	SHOW_FIELD(31, 20, 1, NULL, "Reserved"	);
	return(0);
}



//*****************************************************************************
static int _ttbsr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15, 0, 1, NULL, "Burst Size");
	SHOW_FIELD(31,16, 1, NULL, "Burst Size");
	return(0);
}



//*****************************************************************************
static int _ttctrr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15,  0, 1, NULL, "Trigger Reference");
	SHOW_FIELD(31, 16, 1, NULL, "Trigger Threshold");
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCTLR),		0,	_bctlr_decode,	"Board Control Register"				},
	{ _GSC_REG(ICR),		0,	_icr_decode,	"Interrupt Control Register"			},
	{ _GSC_REG(IDBR),		0,	_idbr_decode,	"Input Data Buffer Register"			},
	{ _GSC_REG(IBCR),		0,	_ibcr_decode,	"Input Buffer Control Register"			},
	{ _GSC_REG(RAGR),		0,	_rgr_decode,	"Rate-A Generator Register"				},
	{ _GSC_REG(RBGR),		0,	_rgr_decode,	"Rate-B Generator Register"				},
	{ _GSC_REG(BUFSR),		0,	_bufsr_decode,	"Buffer Size Register"					},
	{ _GSC_REG(BSTSR),		0,	_bstsr_decode,	"Burst Size Register"					},
	{ _GSC_REG(SSCR),		0,	_sscr_decode,	"Scan & Sync Control Register"			},
	{ _GSC_REG(ACAR),		0,	_acar_decode,	"Active Channel Assignment Register"	},
	{ _GSC_REG(BCFGR),		0,	_bcfgr_decode,	"Board Configuration Register"			},
	{ _GSC_REG(AVR),		0,	NULL,			"Autocal Values Register"				},
	{ _GSC_REG(ARWR),		0,	NULL,			"Auxiliary R/W Register"				},
	{ _GSC_REG(ASIOCR),		0,	_asiocr_decode,	"Auxiliary Sync I/O Control Register"	},
	{ _GSC_REG(SMUWR),		0,	_smwr_decode,	"Scan Marker Upper Word Register"		},
	{ _GSC_REG(SMLWR),		0,	_smwr_decode,	"Scan Marker Lower Word Register"		},
	{ _GSC_REG(LLCR),		0,	_llcr_decode,	"Low Latency Control Register"			},

	// Time Tag Registers
	{ _GSC_REG(TTCR),		0,	_ttcr_decode,	"Time Tag Configuration Register"		},
	{ _GSC_REG(TTACMR),		0,	_ttacmr_decode,	"Time Tag Active Channel Mask Register"	},
	{ _GSC_REG(TTCLR),		0,	_ttclr_decode,	"Time Tag Counter Lower Register"		},
	{ _GSC_REG(TTCUR),		0,	_ttcur_decode,	"Time Tag Counter Upper Register"		},
	{ _GSC_REG(TTRDR),		0,	_ttrdr_decode,	"Time Tag Rate Divider Register"		},
	{ _GSC_REG(TTBSR),		0,	_ttbsr_decode,	"Time Tag Burst Size Register"			},

	{ _GSC_REG(TTC00TRR),	0,	_ttctrr_decode,	"Time Tag Channel 00 Threshold/Reference Register"	},
	{ _GSC_REG(TTC01TRR),	0,	_ttctrr_decode,	"Time Tag Channel 01 Threshold/Reference Register"	},
	{ _GSC_REG(TTC02TRR),	0,	_ttctrr_decode,	"Time Tag Channel 02 Threshold/Reference Register"	},
	{ _GSC_REG(TTC03TRR),	0,	_ttctrr_decode,	"Time Tag Channel 03 Threshold/Reference Register"	},
	{ _GSC_REG(TTC04TRR),	0,	_ttctrr_decode,	"Time Tag Channel 04 Threshold/Reference Register"	},
	{ _GSC_REG(TTC05TRR),	0,	_ttctrr_decode,	"Time Tag Channel 05 Threshold/Reference Register"	},
	{ _GSC_REG(TTC06TRR),	0,	_ttctrr_decode,	"Time Tag Channel 06 Threshold/Reference Register"	},
	{ _GSC_REG(TTC07TRR),	0,	_ttctrr_decode,	"Time Tag Channel 07 Threshold/Reference Register"	},
	{ _GSC_REG(TTC08TRR),	0,	_ttctrr_decode,	"Time Tag Channel 08 Threshold/Reference Register"	},
	{ _GSC_REG(TTC09TRR),	0,	_ttctrr_decode,	"Time Tag Channel 09 Threshold/Reference Register"	},

	{ _GSC_REG(TTC10TRR),	0,	_ttctrr_decode,	"Time Tag Channel 10 Threshold/Reference Register"	},
	{ _GSC_REG(TTC11TRR),	0,	_ttctrr_decode,	"Time Tag Channel 11 Threshold/Reference Register"	},
	{ _GSC_REG(TTC12TRR),	0,	_ttctrr_decode,	"Time Tag Channel 12 Threshold/Reference Register"	},
	{ _GSC_REG(TTC13TRR),	0,	_ttctrr_decode,	"Time Tag Channel 13 Threshold/Reference Register"	},
	{ _GSC_REG(TTC14TRR),	0,	_ttctrr_decode,	"Time Tag Channel 14 Threshold/Reference Register"	},
	{ _GSC_REG(TTC15TRR),	0,	_ttctrr_decode,	"Time Tag Channel 15 Threshold/Reference Register"	},
	{ _GSC_REG(TTC16TRR),	0,	_ttctrr_decode,	"Time Tag Channel 16 Threshold/Reference Register"	},
	{ _GSC_REG(TTC17TRR),	0,	_ttctrr_decode,	"Time Tag Channel 17 Threshold/Reference Register"	},
	{ _GSC_REG(TTC18TRR),	0,	_ttctrr_decode,	"Time Tag Channel 18 Threshold/Reference Register"	},
	{ _GSC_REG(TTC19TRR),	0,	_ttctrr_decode,	"Time Tag Channel 19 Threshold/Reference Register"	},

	{ _GSC_REG(TTC20TRR),	0,	_ttctrr_decode,	"Time Tag Channel 20 Threshold/Reference Register"	},
	{ _GSC_REG(TTC21TRR),	0,	_ttctrr_decode,	"Time Tag Channel 21 Threshold/Reference Register"	},
	{ _GSC_REG(TTC22TRR),	0,	_ttctrr_decode,	"Time Tag Channel 22 Threshold/Reference Register"	},
	{ _GSC_REG(TTC23TRR),	0,	_ttctrr_decode,	"Time Tag Channel 23 Threshold/Reference Register"	},
	{ _GSC_REG(TTC24TRR),	0,	_ttctrr_decode,	"Time Tag Channel 24 Threshold/Reference Register"	},
	{ _GSC_REG(TTC25TRR),	0,	_ttctrr_decode,	"Time Tag Channel 25 Threshold/Reference Register"	},
	{ _GSC_REG(TTC26TRR),	0,	_ttctrr_decode,	"Time Tag Channel 26 Threshold/Reference Register"	},
	{ _GSC_REG(TTC27TRR),	0,	_ttctrr_decode,	"Time Tag Channel 27 Threshold/Reference Register"	},
	{ _GSC_REG(TTC28TRR),	0,	_ttctrr_decode,	"Time Tag Channel 28 Threshold/Reference Register"	},
	{ _GSC_REG(TTC29TRR),	0,	_ttctrr_decode,	"Time Tag Channel 29 Threshold/Reference Register"	},

	{ _GSC_REG(TTC30TRR),	0,	_ttctrr_decode,	"Time Tag Channel 30 Threshold/Reference Register"	},
	{ _GSC_REG(TTC31TRR),	0,	_ttctrr_decode,	"Time Tag Channel 31 Threshold/Reference Register"	},

	{ NULL, 0, 0, 0, 0,		NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ai32ssc1m_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ai32ssc1m_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ai32ssc1m_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ai32ssc1m_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai32ssc1m_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, ai32ssc1m_reg_read);
	return(errs);
}




